<?php 
    /*Feito por Victor Rangel
    Produto Odhen*/
namespace Teknisa\MAN\Controllers;
    
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Services;

class GrupoOperador extends CrudWithParameterBag { 
    
    protected $dataSourceNameArray = array (
        'grupo_operadores' => 'ger02000_grupoOperador',
        'operadores_grupo' => 'ger02000_grupoOperador_operGrupo',
        'permissoes_grupo' => 'ger02000_grupoOperador_permissaoGrupo'
    );
    protected $parameterBagColumnsArray = array (
        'grupo_operadores' =>  array('NRORG', 'CDGRUPOPER'),
        'operadores_grupo' =>  array('NRORG', 'CDGRUPOPER'),
        'permissoes_grupo' =>  array('NRORG', 'CDGRUPOPER'),
    );
    
   public function __construct ($crudService, $parameterBag, Manager $dataSourceManager, Services\GrupoOperadorService $grupoOperadorService) {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->service           = $grupoOperadorService;
    }
    
    ///find's
    public function findGrupoOperadores(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['grupo_operadores'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['grupo_operadores'];
        parent::find($request, $response);
    }
    
    public function findOperadoresPorGrupo(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['operadores_grupo'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['operadores_grupo'];
        parent::find($request, $response);
    }
    
    public function findAutGrupo(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['permissoes_grupo'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['permissoes_grupo'];
        parent::find($request, $response);
    }
    
    ///save's
    public function saveGrupoOperadores(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['grupo_operadores'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['grupo_operadores'];
        self::saveWithoutNotification($request, $response);
    }
    
    public function saveOperadoresPorGrupo(DTO\Request $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        
        $operadores = $row['CDOPERADOR'];
        
        for($i = 0; $i < count($operadores); $i++) {
            $params = array(
                ':CDGRUPOPER'   => $row['CDGRUPOPER'],
    		    ':CDOPERADOR'   => $operadores[$i],
    		    ':CDOPERULTATU' => $row['CDOPERULTATU'],
    		    ':DTINCLUSAO'   => $row['DTINCLUSAO'],
    		    ':NRORG'        => $row['NRORG']
            );
            
            $this->service->insereOperadorGrupo($params);
        }
    }
    
    public function saveAutGrupo(DTO\Request $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $row['IDAUTGRUPOP'] = 'INDF';
        $row['NRAUTGRUPOP'] = $this->service->selectNrAut(array(':NRORG' => $row['NRORG']))[0]['NRAUTGRUPOP'];
        
        if((empty($row['NRAUTGRUPOP'])) || ($row['NRAUTGRUPOP'] == null)) {
            $row['NRAUTGRUPOP'] = 1;
        }
        
        $params = array(
            ':CDGRUPOPER'     => $row['CDGRUPOPER'], 
            ':IDAUTGRUPOP'    => 'INDF', 
            ':NRAUTGRUPOP'    => $row['NRAUTGRUPOP'], 
            ':DTINCLUSAO'     => $row['DTINCLUSAO'], 
            ':CDOPERINCLUSAO' => $row['CDOPERINCLUSAO'], 
            ':NRORG'          => $row['NRORG']
        );
        
        
        $this->service->insereAutorizacao($params);
    }
    
    public function saveWithoutNotification(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    ///delete's
    public function deleteGrupoOperadores(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $dataSet = new DataSet($this->dataSourceNameArray['grupo_operadores'], array($row));
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteOperadoresPorGrupo(DTO\Request $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows();
        
        $params = array(
            ':CDGRUPOPER' => $row[0]['CDGRUPOPER']
        );
        
        for($i = 0; $i < count($row); $i++) {
            $params[':CDOPERADOR'] = $row[$i]['CDOPERADOR'];
            
            $this->service->deleteOperadorGrupo($params);
        }
    }
    
    public function deleteAutGrupo(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $dataSet = new DataSet($this->dataSourceNameArray['permissoes_grupo'], array($row));
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}
?>