<?php
namespace Teknisa\MAN\Services;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;
use \Setforms2\Shared\ORM\DateTime;

class ControleVendasService {
    protected $utilService;
    protected $entityManager;
    
    public function __construct(Utilities\Util $utilService, EntityManager $entityManager) {
        $this->utilService = $utilService;
        $this->entityManager = $entityManager;
    }
    
    // ----- FUNCOES -----
    
    public function calculosRelatorio($conditions, $dataSet, $aux) {
        $cdsRelatorio = [];
        
        $filiais = array_unique(array_column($dataSet, 'CDFILIAL'));
        
        $margemBruta = [];
        for($a = 0; $a < count($filiais); $a++) {
            $margemBruta[$a] = $this->procedureMargemBruta($conditions, $aux, $filiais[$a]);
        }
        
        for($i=0; $i<count($dataSet); $i++ ) {
            $dataSet[$i]['VRCUSTO'] = $this->procedureCustoCompProd($dataSet[$i], $aux); 
            
            for($j = 0; $j < count($margemBruta); $j++) {
                if($dataSet[$i]['CDFILIAL'] == $filiais[$j]) {
                    $dataSet[$i]['MARGEMBRUTA'] = $margemBruta[$j];
                }
            }
            
            $params = array('P_CDEMPRESA' => $conditions['P_CDEMPRESA'], 
            'P_CDFILIAL' => $dataSet[$i]['CDFILIAL'], 
            'P_CDCAIXA' => $conditions['P_CDCAIXA'], 
            'P_TCDCAIXA' => $conditions['P_TCDCAIXA'],
            'P_CDLOJA' => $conditions['P_CDLOJA'],
            'P_TCDLOJA' => $conditions['P_TCDLOJA'],
            'P_CDSUBGRPROD' => $conditions['P_CDSUBGRPROD'],
            'P_TCDSUBGRPROD' => $conditions['P_TCDSUBGRPROD'],
            'P_DTVENDAINI' => $conditions['P_DTVENDAINI'], 
            'P_DTVENDAFIN' => $conditions['P_DTVENDAFIN']);
            
            $adqtotprod = $this->entityManager->getConnection()->fetchAll(Utilities\Query::ADQTOTAISPROD,$params);
            $adqtotprod = $adqtotprod[0]['G'];
            
            if($aux['INFREL'] == 0) { 
                $vrqrdevend = $adqtotprod;       
            } else {
                $params = array('P_CDFILIAL' => $dataSet[$i]['CDFILIAL'], 
                'P_CDCAIXA' => $conditions['P_CDCAIXA'],
                'P_TCDCAIXA' => $conditions['P_TCDCAIXA'],
                'P_CDLOJA' => $conditions['P_CDLOJA'],
                'P_TCDLOJA' => $conditions['P_TCDLOJA'],
                'P_DTVENDAINI' => $conditions['P_DTVENDAINI'],
                'P_DTVENDAFIN' => $conditions['P_DTVENDAFIN']
                );
                $adqredz = $this->entityManager->getConnection()->fetchAll(Utilities\Query::ADQREDUCAOZ, $params);
                $adqredz = $adqredz[0]["NRCUPONS"];
                
                if($adqredz == '') { 
                    $adqredz = 0;
                }
                $vrqrdevend = $adqredz;
            }
            $dtInicial = substr($conditions['P_DTVENDAINI'], 3);
            
            $params = array('P_CDFILIAL' => $dataSet[$i]['CDFILIAL'], 'P_DATAINI' => $dtInicial, 'P_DATAFIM' => $conditions['P_DTVENDAFIN']);
            $adqfrete = $this->entityManager->getConnection()->fetchAll(Utilities\Query::ADQFRETE, $params);
            $adqfrete = $adqfrete[0]['VRFRETE'];
           
            $precoSugerido = $this->retornaPreco($dataSet[$i]['CDFILIAL'], $dataSet[$i]['CDPRODUTO']);          
            
            $codImposto = $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_COD_IMPOSTO, array('P_SGIMPOSTO' => 'PIS'));
            if(!empty($codImposto)) {
                $pisCofins = $this->procedureCalculaImposto($conditions, $dataSet[$i], $codImposto[0]['CDIMPOSTO']);    
            } else {
                $pisCofins = 0.00;
            }
            
            $codImposto = $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_COD_IMPOSTO, array('P_SGIMPOSTO' => 'COFINS'));
            if(!empty($codImposto)) {
                $pisCofins += $this->procedureCalculaImposto($conditions, $dataSet[$i], $codImposto[0]['CDIMPOSTO']);
            } else {
                $pisCofins = 0.00;
            }
            
            $codImposto = $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_COD_IMPOSTO, array('P_SGIMPOSTO' => 'ICMS'));
            if(!empty($codImposto)) {
                $icms = $this->procedureCalculaImposto($conditions, $dataSet[$i], $codImposto[0]['CDIMPOSTO']);
            } else {
                $icms = 0.00;
            }
            
            // armazenar valores para montar um cds final
            $cdsRelatorio[$i]['CDFILIAL'] = $dataSet[$i]['CDFILIAL'];
            $cdsRelatorio[$i]['CDCAIXA'] = $dataSet[$i]['CDCAIXA'];
            $cdsRelatorio[$i]['CDEMPRESA'] = $dataSet[$i]['CDEMPRESA'];
            $cdsRelatorio[$i]['CDLOJA'] = $dataSet[$i]['CDLOJA'];
            $cdsRelatorio[$i]['CDPRODUTO'] = $dataSet[$i]['CDPRODUTO'];
            $cdsRelatorio[$i]['CDSUBGRPROD'] = $dataSet[$i]['CDSUBGRPROD'];
            $cdsRelatorio[$i]['NMSUBGRPROD'] = $dataSet[$i]['NMSUBGRPROD'];
            $cdsRelatorio[$i]['QTPRODVEND'] = $dataSet[$i]['QTPRODVEND'];
            $cdsRelatorio[$i]['VRTOTPERC'] = $dataSet[$i]['VRTOTPERC'];
            $cdsRelatorio[$i]['VRPREBRUTO'] = $dataSet[$i]['VRPREBRUTO'];
            $cdsRelatorio[$i]['VRPRELIQ'] = $dataSet[$i]['VRPRELIQ'];
            $cdsRelatorio[$i]['TOTVRPRELIQ'] = $dataSet[$i]['TOTVRPRELIQ'];
            $cdsRelatorio[$i]['VRDESITVEND'] = $dataSet[$i]['VRDESITVEND'];
            $cdsRelatorio[$i]['NMFILIAL'] = $dataSet[$i]['NMFILIAL'];
            $cdsRelatorio[$i]['NMPRODUTO'] = $dataSet[$i]['NMPRODUTO'];
            $cdsRelatorio[$i]['SGUNIDADE'] = $dataSet[$i]['SGUNIDADE'];
            $cdsRelatorio[$i]['NMEMPRESA'] = $dataSet[$i]['NMEMPRESA'];
            $cdsRelatorio[$i]['NRCUSTOPROD'] = $aux['NMTIPOCUSTO']; 
            $cdsRelatorio[$i]['PCUSTO'] = $dataSet[$i]['VRCUSTO']; 
            $cdsRelatorio[$i]['VRQTDEVEND'] = $vrqrdevend; 
            $cdsRelatorio[$i]['IMP'] = $dataSet[$i]['IMP'] * $dataSet[$i]['QTPRODVEND'];
            $cdsRelatorio[$i]['DTVENDAINI'] = $conditions['P_DTVENDAINI'];
            $cdsRelatorio[$i]['DTVENDAFIN'] = $conditions['P_DTVENDAFIN'];
            $cdsRelatorio[$i]['VREMARBRUTA'] = $dataSet[$i]['MARGEMBRUTA']; 
            $cdsRelatorio[$i]['VRIMPOSTO'] = ($icms * $dataSet[$i]['QTPRODVEND']) - ($pisCofins * $dataSet[$i]['QTPRODVEND']) ; 
            $cdsRelatorio[$i]['PRECOSUGER'] = $precoSugerido;
            $cdsRelatorio[$i]['ICMS'] = $icms;
            $cdsRelatorio[$i]['PISCOFINS'] = $pisCofins;
            
            if($cdsRelatorio[$i]['VRIMPOSTO'] < 0) {
                $cdsRelatorio[$i]['VRIMPOSTO'] = 0;    
            }
            $cdsRelatorio[$i]['VRPISCOFINS'] = $pisCofins * $dataSet[$i]['QTPRODVEND'];
            $cdsRelatorio[$i]['VRFRETE'] = $adqfrete;
            $cdsRelatorio[$i]['NMCAIXA_OLD'] = $dataSet[$i]['NMCAIXA'];
        }  
        
        if(!isset($cdsRentabilidade)) {
            $cdsRentabilidade = array();
            $cdsRentabilidade[0]['NMPRODUTO'] = '';
        }
        
        return $this -> geraRelatorio($cdsRelatorio, $cdsRentabilidade);
    }
    
    private function geraRelatorio($cdsRelatorio, $cdsRentabilidade) {
        $stProdutoAux = '';
        $index = 0;
        $primeira = true;
        for($i = 0; $i < count($cdsRelatorio); $i++) {
            if($cdsRentabilidade[$index]['NMPRODUTO'] != $cdsRelatorio[$i]['NMPRODUTO']) {
                if($i != 0) {
                    $index++;
                }
                
                $cdsRentabilidade[$index]['CDFILIAL'] = $cdsRelatorio[$i]['CDFILIAL'];
                $cdsRentabilidade[$index]['CDCAIXA'] = $cdsRelatorio[$i]['CDCAIXA'];
                $cdsRentabilidade[$index]['CDEMPRESA'] = $cdsRelatorio[$i]['CDEMPRESA'];
                $cdsRentabilidade[$index]['CDLOJA'] = $cdsRelatorio[$i]['CDLOJA'];
                $cdsRentabilidade[$index]['CDPRODUTO'] = $cdsRelatorio[$i]['CDPRODUTO'];
                $cdsRentabilidade[$index]['CDSUBGRPROD'] = $cdsRelatorio[$i]['CDSUBGRPROD'];
                $cdsRentabilidade[$index]['NMSUBGRPROD'] = $cdsRelatorio[$i]['NMSUBGRPROD'];
                $cdsRentabilidade[$index]['QTPRODVEND'] = $cdsRelatorio[$i]['QTPRODVEND'];
                $cdsRentabilidade[$index]['VRTOTPERC'] = $cdsRelatorio[$i]['VRTOTPERC'];
                $cdsRentabilidade[$index]['VRPREBRUTO'] = $cdsRelatorio[$i]['VRPREBRUTO'];
                $cdsRentabilidade[$index]['VRPRELIQ'] = $cdsRelatorio[$i]['VRPRELIQ'];
                $cdsRentabilidade[$index]['TOTVRPRELIQ'] = $cdsRelatorio[$i]['TOTVRPRELIQ'];
                $cdsRentabilidade[$index]['VRDESITVEND'] = $cdsRelatorio[$i]['VRDESITVEND'];
                $cdsRentabilidade[$index]['VRDESITVEND'] = $cdsRelatorio[$i]['VRDESITVEND'];
                $cdsRentabilidade[$index]['NMPRODUTO'] = $cdsRelatorio[$i]['NMPRODUTO'];
                $cdsRentabilidade[$index]['SGUNIDADE'] = $cdsRelatorio[$i]['SGUNIDADE'];
                $cdsRentabilidade[$index]['NMEMPRESA'] = $cdsRelatorio[$i]['NMEMPRESA'];
                $cdsRentabilidade[$index]['NRCUSTOPROD'] = $cdsRelatorio[$i]['NRCUSTOPROD'];
                $cdsRentabilidade[$index]['PCUSTO'] = $cdsRelatorio[$i]['PCUSTO'];
                $cdsRentabilidade[$index]['VRQTDEVEND'] = $cdsRelatorio[$i]['VRQTDEVEND'];
                $cdsRentabilidade[$index]['IMP'] = $cdsRelatorio[$i]['IMP'];
                $cdsRentabilidade[$index]['DTVENDAINI'] = $cdsRelatorio[$i]['DTVENDAINI'];
                $cdsRentabilidade[$index]['DTVENDAFIN'] = $cdsRelatorio[$i]['DTVENDAFIN'];
                $cdsRentabilidade[$index]['VREMARBRUTA'] = $cdsRelatorio[$i]['VREMARBRUTA'];
                $cdsRentabilidade[$index]['VRIMPOSTO'] = $cdsRelatorio[$i]['VRIMPOSTO'];
                $cdsRentabilidade[$index]['VRPISCOFINS'] = $cdsRelatorio[$i]['VRPISCOFINS'];
                $cdsRentabilidade[$index]['VRFRETE'] = $cdsRelatorio[$i]['VRFRETE'];
                $cdsRentabilidade[$index]['NMCAIXA_OLD'] = $cdsRelatorio[$i]['NMCAIXA_OLD'];
                $cdsRentabilidade[$index]['PRECOSUGER'] = $cdsRelatorio[$i]['PRECOSUGER'];
                $cdsRentabilidade[$index]['ICMS'] = $cdsRelatorio[$i]['ICMS'];
                $cdsRentabilidade[$index]['PISCOFINS'] = $cdsRelatorio[$i]['PISCOFINS'];
                $cdsRentabilidade[$index]['VRIMPOSTO'] = ($cdsRentabilidade[$index]['ICMS'] * $cdsRentabilidade[$index]['QTPRODVEND']) - ($cdsRentabilidade[$index]['PISCOFINS'] * $cdsRentabilidade[$index]['QTPRODVEND']); 
                $cdsRentabilidade[$index]['MUVALOR'] = $cdsRentabilidade[$index]['VRPRELIQ'] - $cdsRentabilidade[$index]['PCUSTO'];
                if($cdsRentabilidade[$index]['VRPRELIQ'] > 0) {
                   $cdsRentabilidade[$index]['MUPORC'] = $cdsRentabilidade[$index]['MUVALOR'] / $cdsRentabilidade[$index]['VRPRELIQ'] * 100; 
                } else {
                    $cdsRentabilidade[$index]['MUPORC'] = 0;
                }
                
                $somaValoresImposto = array_sum(array_column($cdsRelatorio, 'VRIMPOSTO'));
                $cdsRentabilidade[$index]['VLVALOR'] = $cdsRentabilidade[$index]['QTPRODVEND'] * $cdsRentabilidade[$index]['VRPRELIQ'];
                if(($cdsRentabilidade[$index]['TOTVRPRELIQ'] - $somaValoresImposto) != 0) {
                    $cdsRentabilidade[$index]['VLPORC'] = $cdsRentabilidade[$index]['VLVALOR'] / ($cdsRentabilidade[$index]['TOTVRPRELIQ'] - $somaValoresImposto);
                } else {
                    $cdsRentabilidade[$index]['VLPORC'] = 0;
                }
                
                $cdsRentabilidade[$index]['MBVALOR'] = ($cdsRentabilidade[$index]['QTPRODVEND'] * $cdsRentabilidade[$index]['VRPREBRUTO']) - ($cdsRentabilidade[$index]['QTPRODVEND'] - $cdsRentabilidade[$index]['PCUSTO']);
                if($cdsRentabilidade[$index]['VREMARBRUTA'] != 0) {
                    $cdsRentabilidade[$index]['MBPORC'] = $cdsRentabilidade[$index]['MBVALOR'] * 100 / $cdsRentabilidade[$index]['VREMARBRUTA'];
                } else {
                    $cdsRentabilidade[$index]['MBPORC'] = 0;
                }
                
                $stProdutoAux  = $cdsRentabilidade[$index]['CDPRODUTO'];
            } else {
                $cdsRentabilidade[$index]['QTPRODVEND'] = floatval($cdsRentabilidade[$index]['QTPRODVEND'] + $cdsRelatorio[$i]['QTPRODVEND']); 
            }
        }
        
        return $cdsRentabilidade;
    }
    
    private function retornaPreco ($stFilial, $stProduto) {
        //Buscar tabela de preço parametrizada para filial
        $params = array('P_CDFILIAL' => $stFilial);
        $adqParaVend = $this->entityManager->getConnection()->fetchAll(Utilities\Query::ADQPARAVEND,$params);
    
        if ($adqParaVend[0]['CDTABEPREC'] == '') {
            $stPrecoSuger = 0.00;
        }  
        else {
            $stCdTabePrec = $adqParaVend[0]['CDTABEPREC']; 
            
            // Buscar vigência da tabela
            $params = array('P_CDFILIAL' => $stFilial, 'P_CDTABEPREC' => $stCdTabePrec, 'P_DATA' => date('d/m/Y'));
            $adqTabeVend = $this->entityManager->getConnection()->fetchAll(Utilities\Query::ADQTABEVEND,$params);
            
            $dtDataVigente = '';
            
            if(empty($adqTabeVend[0]['DTINIVGPREC'])) {
                echo 'Não há Tabela de Preço Vigente.';
            } else {
                $dtDataVigente = $adqTabeVend[0]['DTINIVGPREC'];
            }
            
            // Buscar preço sugerido
            $params = array('P_CDFILIAL' => $stFilial, 'P_CDTABEPREC' => $stCdTabePrec, 'P_DTINIVGPREC' => $dtDataVigente, 'P_CDPRODUTO' => $stProduto );
            $adqItemPreco = $this->entityManager->getConnection()->fetchAll(Utilities\Query::ADQITEMPRECO,$params);
            $stPrecoSuger = $adqItemPreco[0]['PRECOSUGER'];
    
            return $stPrecoSuger;
        }
    }
    
    private function separaArray($array) {
        $array = explode('_', $array);
        array_shift($array); array_pop($array);
        return $array;
    }
    
    // ----- PROCEDURES -----
    
    private function procedureCustoCompProd($row, $aux) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CUSTO_COMP_PROD');
        $procedure->addParam(new Param('P_CDPRODUTO', Param::PARAM_INPUT, $row['CDPRODUTO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIPROC', Param::PARAM_INPUT, $row["CDFILIAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOCUSTO', Param::PARAM_INPUT, $aux['NMTIPOCUSTO'], Param::PARAM_TYPE_INT)); 
        $procedure->addParam(new Param('P_QTPRODUTO', Param::PARAM_INPUT, 1, Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_LOJA', Param::PARAM_INPUT, $row['CDLOJA'], Param::PARAM_TYPE_STR)); 
        $procedure->addParam(new Param('P_VRCUSTO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_INT, 100));
        $retorno = $procedure->execute();
        $retorno = floatval($retorno['P_VRCUSTO']);
        return $retorno;
    }
    
    private function procedureMargemBruta($conditions, $aux, $filial) {
        $dtInicial = DateTime::createFromFormat('d/m/Y', $conditions['P_DTVENDAINI']);
        $dtFinal   = DateTime::createFromFormat('d/m/Y', $conditions['P_DTVENDAFIN']);
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'MARGEM_BRUTA');
        $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $conditions['P_CDEMPRESA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTVENDAINI' , Param::PARAM_INPUT, $dtInicial->format('d/m/Y'), Param::PARAM_TYPE_STR)); 
        $procedure->addParam(new Param('P_DTVENDAFIN', Param::PARAM_INPUT, $dtFinal->format('d/m/Y'), Param::PARAM_TYPE_STR)); 
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIPROC', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCAIXA', Param::PARAM_INPUT, $conditions['P_CDCAIXA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TCDCAIXA', Param::PARAM_INPUT, $conditions['P_TCDCAIXA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOJA', Param::PARAM_INPUT, $conditions['P_CDLOJA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TCDLOJA', Param::PARAM_INPUT, $conditions['P_TCDLOJA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSUBGRPROD', Param::PARAM_INPUT, $conditions['P_CDSUBGRPROD'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TCDSUBGRPROD', Param::PARAM_INPUT, $conditions['P_TCDSUBGRPROD'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOCUSTO', Param::PARAM_INPUT, $aux['NMTIPOCUSTO'], Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_MARGEMBRUTA', Param::PARAM_OUTPUT, null,  Param::PARAM_TYPE_INT, 20));
        
        $retorno = $procedure->execute();
        $retorno = floatval($retorno['P_MARGEMBRUTA']);
        return $retorno;
    }
    
    private function procedureCalculaImposto($conditions, $row, $cdtipoimposto) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CALCULA_IMPOSTO');
        $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $conditions['P_CDEMPRESA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTO', Param::PARAM_INPUT, $row['CDPRODUTO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row['CDCLIENTE'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDINSCESTA', Param::PARAM_INPUT, $row['CDINSCESTA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPREBRUTO', Param::PARAM_INPUT, $row['VRPREBRUTO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTVENDAINI', Param::PARAM_INPUT, $conditions['P_DTVENDAINI'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTVENDAFIN', Param::PARAM_INPUT, $conditions['P_DTVENDAFIN'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDIMPOSTO', Param::PARAM_INPUT, $cdtipoimposto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRIMPOSTO', Param::PARAM_OUTPUT, null,  Param::PARAM_TYPE_INT, 20));
        
        $retorno = $procedure->execute();
        $retorno = floatval($retorno['P_VRIMPOSTO']);
        
        return $retorno;
    }
}
    
